/****************************************************************************

 *																			*

 * THIS SOFTWARE IS PROVIDED AS-IS WITH NO WARRANTIES EXPRESSED OR IMPLIED	*

 * THE AUTHOR WILL NOT BE RESPONSIBLE FOR ANY DAMAGE DERIVED BY USE OR		*

 * MISUSE OF THIS SOFTWARE OR PARTS OF IT									*

 *																			*

 * THIS SOFTWARE IS UNDER THE BSD LICENSE			 						*

 *																			*

 * comments and bug reports are welcome, mail to dario@dsi.unimi.it			*

 *																			*

 ****************************************************************************/



import java.io.*;

import mmap.AddressSpace;



/** This class tests the swap module.

  * <br>

  * <ol>

  * <li>First of all an empty space is created (test is the name).</li>

  * <li>A series of zeros is inserted.<br>

  *     the user should see only the index grow.</li>

  * <li>A file is placed inside the swap at a random position.<br>

  *     now the user should see the data grow.</li>

  * <li>The file is extracted and placed is the local directory.</li>

  * <li>A part of the file is blanked.<br>

  *     the user should see that the data file partially reduce.</li>

  * <li>The rest of the file is blanked.<br>

  *     the data file should reduce to 1024 bytes again.</li>

  * 

  * @version	 1.0 June 05 2000

  * @author	 Dario Maggiorini (dario@dsi.unimi.it)

  */  



public class SwapTest {



	/** Noone will instatntiate me */

	

    private SwapTest() {}

    

	/** Just call it.

	  * <br>

	  * No command line parameters are needed.<br>

	  */

    public static final void main(String args[]) {

    	

    	int		dumpSize = 0;

		int		aRandomPlace = (int) (java.lang.Math.random() * 2000000.0);

		String	dumpInFile = "/kernel";

		String	dumpOutFile = "dump";

    	

		// create an empty space



        System.out.print("creating   <empty>");

        try {

            AddressSpace.createSpace("test");

        }

        catch (Exception e) {

            e.printStackTrace();

        }

        System.out.print("\n");



		// write some zeroes around, only index should grow



        System.out.print("blanking   <0 -> " + aRandomPlace + " step 1000>");

        try {

            AddressSpace as = new AddressSpace("test");

            for (int i = 0; i < aRandomPlace; i += 1000) {

                as.write(i, (byte) 0x00);

            }

            as.close();

        }

        catch (Exception e) {

            e.printStackTrace();

        }

    	System.out.print("\n");



		// now put a file inside our swap space

		// starting at position "aRandomPlace"



		System.out.print("insert which file ? >");

		try {

			byte[] buffer = new byte[1024];

			int len = System.in.read(buffer, 0, 1024);

			dumpInFile = new String(buffer, 0, len - 2);			

		}

		catch (Exception e) {

			e.printStackTrace();

		}

		

        System.out.print("inserting  <" + dumpInFile + " " 

							+ aRandomPlace + " -> ? >");

        try {

            AddressSpace as = new AddressSpace("test");

            byte buffer[] = new byte[10000];

            int pointer = aRandomPlace;



            FileInputStream fis = new FileInputStream(dumpInFile);

            

            int l;

            while((l = fis.read(buffer)) == buffer.length) {

                dumpSize += l;

                as.write(pointer, buffer);

                pointer += l;

            }

            dumpSize += l;

            as.write(pointer, buffer, l);

            fis.close();

            as.close();

        }

        catch (Exception e) {

            e.printStackTrace();

        }

        System.out.print("\n");

        

		// now extract the same file and put it in a file called "dumpOutFile"



        System.out.print("retrieving <" + dumpOutFile + " " 

							+ aRandomPlace + " -> "

							+ ( aRandomPlace + dumpSize ) + " >");

        try {

            AddressSpace as = new AddressSpace("test");

            byte buffer[] = new byte[1000];

            int pointer = aRandomPlace;



            FileOutputStream fos = new FileOutputStream(dumpOutFile, false);

            

            int l;

			int i = dumpSize;

            while (i > 0) {

                l = as.read(pointer, buffer);

                fos.write(buffer, 0, i < l ? i : l);

                pointer += l;

                i -= l;

            }

            fos.close();

            as.close();

        }

        catch (Exception e) {

            e.printStackTrace();

        }

        System.out.print("\n");



		// wiping part of the file

		// only reduced amount of data file should remain



        System.out.print("wiping     <" 

							+ aRandomPlace + " -> "

							+ ( aRandomPlace + 1000000 ) + " >");

        try {

            AddressSpace as = new AddressSpace("test");

			int i;

            for (i = aRandomPlace; i < aRandomPlace + 1000000; i += 1) {

                as.write(i, (byte) 0x00);

            }

            as.close();

        }

        catch (Exception e) {

            e.printStackTrace();

        }

        System.out.print("\n");    



		// wiping all the file

		// data file should be reduced to 1024 bytes again



        System.out.print("cleaning   <" 

							+ aRandomPlace + " -> "

							+ ( aRandomPlace + dumpSize ) + " >");

        try {

            AddressSpace as = new AddressSpace("test");



			int pointer = aRandomPlace;

			int i = dumpSize;

        	while (i > 0) {

                as.write(pointer, (byte) 0x00);

				pointer += 1;

				i -= 1;

            }

            as.close();

        }

        catch (Exception e) {

            e.printStackTrace();

        }

        System.out.print("\n");    

    }

}

